package cn.com.do1.component.bi.gateway.config.DynamicRoute;

import cn.com.do1.component.bi.gateway.filter.ReqDecryptFilter;
import cn.com.do1.component.bi.gateway.filter.RespEncryptFilter;
import cn.hutool.core.util.ObjectUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.gateway.filter.GatewayFilter;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.cloud.gateway.route.Route;
import org.springframework.cloud.gateway.route.RouteLocator;
import org.springframework.cloud.gateway.route.builder.GatewayFilterSpec;
import org.springframework.cloud.gateway.route.builder.RouteLocatorBuilder;
import org.springframework.cloud.gateway.route.builder.UriSpec;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.HttpCookie;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.function.Function;

import static org.springframework.cloud.gateway.support.ServerWebExchangeUtils.GATEWAY_ROUTE_ATTR;

@Configuration
public class GatewayConfig {

    @Autowired
    private DynamicRouteConfig dynamicRouteConfig;

    @Bean
    public RouteLocator headerRoutes(RouteLocatorBuilder builder){
        RouteLocatorBuilder.Builder routes = builder.routes();
        List<RouteInfo> routeInfos = dynamicRouteConfig.getRouteInfos();
        if (ObjectUtil.isNotEmpty(routeInfos)){
            DynamicRouteConfigConverter.convertToTenantIdToInstances(routeInfos);
            for (RouteInfo routeInfo:routeInfos){
                routes.route(routeInfo.getId(), r ->{
                    return r.path(routeInfo.getPredicatesPaths().get(0))
                            .filters(new Function<GatewayFilterSpec, UriSpec>() {
                                @Override
                                public UriSpec apply(GatewayFilterSpec gatewayFilterSpec) {
                                    Map<String, String> rewritePathMap = routeInfo.getRewritePathMap();
                                    if (ObjectUtil.isNotEmpty(rewritePathMap)) {
                                        gatewayFilterSpec.rewritePath(rewritePathMap.get("regexp"),rewritePathMap.get("replacement"));
                                    }
                                    return gatewayFilterSpec.filter(new HeaderToPathGatewayFilter(routeInfo.getId()))
                                            .filter(new ReqDecryptFilter(-25))
                                            .filter(new RespEncryptFilter(-23));
                                }
                            }).uri(routeInfo.getDefaultUri());
                });

            }
        }
        return routes.build();
    }


    /**
     * 租户路由过滤器
     */
    public class HeaderToPathGatewayFilter implements GatewayFilter{
        private String routeInfoId;

        public HeaderToPathGatewayFilter(String routeInfoId) {
            this.routeInfoId = routeInfoId;
        }

        @Override
        public Mono<Void> filter(ServerWebExchange exchange, GatewayFilterChain chain) {
            List<HttpCookie> tenantIdCookies = exchange.getRequest().getCookies().get("tenantId");
            if (ObjectUtil.isNotEmpty(tenantIdCookies)){
                String tenantId = tenantIdCookies.get(0).getValue();
                if (ObjectUtil.isEmpty(tenantId)){
                    Map<String, List<String>> tenantIdToInstances = DynamicRouteConfigConverter.getTenantIdToInstances();
                    String instanceTenantIdKey =routeInfoId + tenantId;
                    if (tenantIdToInstances.containsKey(instanceTenantIdKey)) {
                        List<String> targetUris = tenantIdToInstances.get(instanceTenantIdKey);
                        int asInt = new Random().ints(0, targetUris.size()).findFirst().getAsInt();
                        Route route = exchange.getAttribute(GATEWAY_ROUTE_ATTR);
                        Route newRoute = Route.async().asyncPredicate(route.getPredicate()).filters(route.getFilters()).id(route.getId())
                                .order(route.getOrder()).uri(targetUris.get(asInt)).build();
                        exchange.getAttributes().put(GATEWAY_ROUTE_ATTR,newRoute);
                    }
                }
            }
            return chain.filter(exchange);
        }
    }


}
